import secrets

from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import AdminPasswordChangeForm
from django.contrib.auth.admin import UserAdmin

admin.site.unregister(get_user_model())


def generate_password():
    return secrets.token_urlsafe(nbytes=7)


def add_help_text_with_generated_password(fields):
    if 'password1' in fields:
        fields['password1'].help_text = 'A randomly generated password you can use: ' + generate_password()


class CustomAdminPasswordChangeForm(AdminPasswordChangeForm):
    def __init__(self, user, *args, **kwargs):
        super().__init__(user, *args, **kwargs)
        add_help_text_with_generated_password(self.fields)


@admin.register(get_user_model())
class CustomUserAdmin(UserAdmin):
    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        form.base_fields['username'].help_text = 'Required. 50 characters or fewer. Letters, digits and @/./+/-/_ only.'
        add_help_text_with_generated_password(form.base_fields)
        return form

    change_password_form = CustomAdminPasswordChangeForm
