from dataclasses import dataclass
from datetime import datetime
from enum import StrEnum


class DataFormat(StrEnum):
    WINDPRO_METEO_EXPORT = "windPRO meteo export"
    WINDPRO_MESORES = "windPRO mesores"
    PLAINTEXT_TIMESERIES_DATA = "Plaintext timeseries data"


@dataclass
class Dataset:
    id: int
    name: str
    description: str
    main_height: float
    first_sample: datetime
    last_sample: datetime
    created: datetime
    created_by: str
    modified: datetime
    modified_by: str
    data_format: DataFormat


@dataclass
class Location:
    """A geographical location associated with meteo data stored in one or more datasets."""
    name: str
    lon: float
    lat: float
    elevation: float
    datasets: list[Dataset]


@dataclass
class Project:
    """A grouping of one or more locations."""
    name: str
    locations: list[Location]
