from django.contrib.auth.models import User
from django.urls import reverse
from rest_framework import status
from rest_framework.test import APITestCase


class MeteoAPITests(APITestCase):
    fixtures = ['auth.user.json']

    def test_get_projects_while_unauthorized_should_return_error(self):
        path = reverse('projects')
        response = self.client.get(path)
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_get_projects_should_return_some_data(self):
        user = User.objects.get(username='test')
        self.client.force_authenticate(user=user)
        path = reverse('projects')
        response = self.client.get(path)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        # Spot check some properties in the returned data
        project = response.data[0]
        self.assertIn('name', project)
        self.assertIn('locations', project)
        location = project['locations'][0]
        self.assertIn('name', location)
        self.assertIn('datasets', location)
        dataset = location['datasets'][0]
        self.assertIn('name', dataset)

    def test_dataset_download_should_return_a_file_response(self):
        user = User.objects.get(username='test')
        self.client.force_authenticate(user=user)
        path = reverse('dataset-download', kwargs={'dataset_id': 1})
        response = self.client.get(path)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertGreater(int(response.headers.get('Content-Length', '0')), 0)
        self.assertIn('Content-Disposition', response.headers)
