from io import BytesIO
from zipfile import ZipFile

from django.http import FileResponse
from django.http import Http404
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.views import APIView

from example_data.example_projects import EXAMPLE_PROJECTS
from .dataclasses import DataFormat
from .serializers import IntegerSerializer
from .serializers import ProjectSerializer


class RootView(APIView):
    """Hello, and welcome to the browsable API index page."""

    # This view is included as a convenience. It is not used by windPRO and can therefore be removed if desired.

    name = 'Meteo API'

    def get(self, request):
        return Response({
            'admin': reverse('admin:index', request=request),
            'api-versions': reverse('api-versions', request=request),
            'projects': reverse('projects', request=request),
            'datasets': reverse('dataset-download', request=request, kwargs=dict(dataset_id=0)),
        })


class VersionListView(APIView):
    """Return the API versions supported by the server."""

    # This view is public because the client needs to know the API version to know the authentication scheme
    authentication_classes = []
    permission_classes = []

    def get(self, request):
        supported_versions = [1]
        serializer = IntegerSerializer(supported_versions, many=True)
        return Response(serializer.data)


class ProjectListView(APIView):
    """Return metadata about all Projects with their Locations and Datasets."""

    def get(self, request):
        projects = EXAMPLE_PROJECTS
        serializer = ProjectSerializer(projects, many=True)
        return Response(serializer.data)


class DatasetDownloadView(APIView):
    """Return a FileResponse containing timeseries data belonging to one Dataset."""

    def get(self, request, dataset_id):

        data_format = ''
        for project in EXAMPLE_PROJECTS:
            for location in project.locations:
                for dataset in location.datasets:
                    if dataset.id == dataset_id:
                        data_format = dataset.data_format

        if data_format == DataFormat.WINDPRO_METEO_EXPORT:
            return FileResponse(open('example_data/meteo_export_example_01.txt', 'rb'))

        if data_format == DataFormat.WINDPRO_MESORES:
            return FileResponse(open('example_data/mesores_example_01.mesores', 'rb'))

        if data_format == DataFormat.PLAINTEXT_TIMESERIES_DATA:
            buffer = BytesIO()
            with ZipFile(buffer, mode='w') as zipfile:
                zipfile.write('example_data/timeseries_example_01.txt')
                zipfile.write('example_data/timeseries_example_02.txt')
            buffer.seek(0)
            return FileResponse(buffer)
            # FileResponse will close the buffer after use

        raise Http404
